package tda551;

public class Signal {

  private enum Speed {
    SUPERSLOW, SLOW, NORMAL, FAST, SUPERFAST;

    private static Speed[] vals = values();

    public Speed up() {
      return vals[Math.min(this.ordinal() + 1, vals.length - 1)];
    }

    public Speed down() {
      return vals[Math.max(this.ordinal() - 1, 0)];
    }
  }

  private SignalStatus status = SignalStatus.RED;
  private Speed speed = Speed.NORMAL;
  private SignalView view = new SignalView();

  public void speedUp() {
    speed = speed.up();
  }

  public void slowDown() {
    speed = speed.down();
  }

  public void run() {
    while (speed != null) {
      try {
        longWait();
        nextStatus();
        shortWait();
        nextStatus();
      } catch (InterruptedException e) {
        speed = null;
      }
    }
  }

  private void longWait() throws InterruptedException {
    Thread.sleep(10000 / (speed.ordinal() + 1));
  }

  private void shortWait() throws InterruptedException {
    Thread.sleep(1000);
  }

  private void nextStatus() {
    status = status.next();
    view.update(status);
  }
}
